---
--[[ This page will provide documentation for the syntax of a "rotation definition 
file". A rotation definition file is valid Lua, and therefore all of the
control structures and other language features are valid in a definition file.
However, there are some extra functions that have been added in order to make
life a little bit easier for people with limited knowledge of Lua. This page 
will show you the syntax of these extra functions and what they mean.

All definition files follow a basic layout and this layout must be followed.
@class file
--]]

FSHelper = {};
---
--[[ Checks whether or not the player is in combat based on whether or not the 
UI is locked down.
@return 1 if the player is in combat, nil otherwise (1nil)
--]]
function FSHelper:InCombat()
    return InCombatLockdown(); 
end;

---
--[[ Checks to see whether or not the player has a pet that shows the pet bar,
such as Unholy Death Knight ghouls or warlock demons.
@return 1 if the player has a pet, nil otherwise (1nil)
--]] 
function FSHelper:HasPet()
    hasPetUI = HasPetUI();
    return hasPetUI;
end;

---
--[[ Checks the health of the player's pet.
@return Percentage health of the player's pet, 0 if it is dead, and nil if it doesn't exist.
--]]
function FSHelper:PetHealth()
    if (UnitExists("pet") and not UnitIsDead("pet")) then
        local currHP = UnitHealth("pet");
        local maxHP = UnitHealthMax("pet");
        local result = math.ceil((currHP / maxHP) * 100);
    elseif (UnitIsDead("pet")) then
        local result = 0;
    else
        local result = nil;
    end;
    return result;
end;

---
--[[ Checks whether or not the player's target has the specified buff.
@param name The name of the buff being checked for.
@param checkme <i>OPTIONAL</i> If we should check whether or not the player casted 
the spell.
@return 1 if the target has the buff (and it was casted by the player if
checkme was not nil), nil otherwise. (1nil).
--]]
function FSHelper:TargetHasBuff(name, checkme)
    if (UnitExists("target")) then
        local bname, _, _, _, _, _, _, caster, _ = UnitBuff("target", name);
    end;
    if (checkme) then
        if (bname == name and caster == player) then
            return 1;
        else
            return nil;
        end;
    else
        if (bname == name) then 
            return 1;
        else
            return nil;
        end;
    end;
end;

---
--[[ Checks whether or not the player's target has the specified debuff.
@param name The name of the buff being checked for.
@param checkme <i>OPTIONAL</i> If we should check whether or not the player casted 
the spell.
@return 1 if the target has the debuff (and it was casted by the player if
checkme was not nil), nil otherwise. (1nil)
--]]
local function getTargetDebuff(name, checkme)
    if (UnitExists("target")) then
        local bname, _, _, _, _, _, _, caster, _ = UnitDebuff("target", name);
    end;
    if checkme then
        if (bname == name and caster == player) then
            return 1;
        else
            return nil;
        end;
    else
        if (bname == name) then
            return 1;
        else
            return nil;
        end;
    end;
end;

---
--[[ Checks whether or not the player's target has the specified debuff AND it was
cast by the player. This is an alias for getTargetDebuff(name, 1).
@param name The debuff to be checked.
@return 1 if the target has the debuff and it was cast by the player, nil
otherwise. (1nil)
@see getTargetDebuff()
--]]
function FSHelper:TargetHasDebuffByMe(name)
    return getTargetDebuff(name, 1);
end;

---
--[[ Checks for the presence of a debuff, regardless of the source
@param name The debuff to check for.
@return 1 if the target has the debuff, nil otherwise (1nil)
--]]
function FSHelper:TargetHasDebuff(name)
    return getTargetDebuff(name);
end;

---
--[[ Chceks for the presence of a buff on the player.
@param name The buff to check for 
@param checkme <i>OPTIONAL</i> If set, also checks if buff was cast by player
@return 1 if the player has the buff, nil otherwise (1nil)
--]]
local function getPlayerBuff(name, checkme)
    bname, _, _, _, _, _, _, caster, _ = UnitBuff("player", name);
    if checkme then
        if bname == name and caster == "player" then
            return 1;
        else
            return nil;
        end;
    else
        if bname == name then
            return 1;
        else
            return nil;
        end;
    end; 
end;

---
--[[ Checks whether or not the player has the specified buff.
@param name The name of the buff being checked for.
@return 1 if the player has the buff, nil otherwise. (1nil)
--]]
function FSHelper:PlayerHasBuff(name)
    return getPlayerBuff(name);
end;

---
-- Checks if the player has the specified buff and it was cast on themselves.
-- @param name The name of the buff to check.
-- @return 1 if the player cast the buff on themself, nil otherwise (1nil)
function FSHelper:PlayerHasBuffByMe(name)
    return getPlayerBuff(name, 1);
end;

---
--[[ Checks whether or not the player has the specified debuff.
@param name The name of the debuff being checked for.
@return 1 if the target has the debuff, nil otherwise (1nil)
--]]
function FSHelper:PlayerHasDebuff()
end;

---
--[[ Checks if the player can cast the specified spell.
@param name The name of the spell being checked.
@return 1 if the spell is castable, nil otherwise (1nil)
--]]
function FSHelper:Castable(name)
    local castable, mana = IsUsableSpell(spell);
    if (not castable or mana == 1) then
        return nil;
    end
    local start, duration, enabled = GetSpellCooldown(spell);
    if (duration == 0) then
        return 1;
    else
        return nil;
    end
end;

---
--[[ Checks whether or not the player has the specified glyph in their current
talent spec.
@param name The name of the glyph being checked for without the "Glyph of"
in front of it, (i.e. Glyph of Souls -> "Souls")
@return 1 if the glyph is inscribed, nil otherwise. (1nil)
--]]
function FSHelper:HasGlyph()
    for i = 1, 6 do
        local _, _, glyphSpell, _ = GetGlyphSocketInfo(i);
        if (glyphSpell ~= nil) then
            local gname,_,_,_,_,_,_,_,_ = GetSpellInfo(glyphSpell);
            if (gname == name) then
                return true;
            end;
        end;
    end;
    return false;
end;

function FSHelper:Alternative()
end;

function FSHelper:Cliptime()
end;

---
--[[ Checks to see if the player knows a particular spell.
@param name The name of the spell to check.
@return 1 if the spell is known, nil otherwise (1nil)
--]]
function FSHelper:KnowSpell(name)
    local status = GetSpellInfo(name)
    if status then
        return 1;
    else
        return nil;
    end;
end;

function FSHelper:HasTalent(name)
end;

function FSHelper:LastCast()
end;
---
-- @return The target's health as a percentage value.
function FSHelper:TargetHealth()
    local percentage = (UnitHealth("target") / UnitHealthMax("target")) * 100;
    return percentage;
end;

---
-- @return The player's health as a percentage value.
function FSHelper:PlayerHealth()
    local percentage = (UnitHealth("player") / UnitHealthMax("player")) * 100;
    return percentage;
end;

---
--[[ Returns the amount of threat that the player has against their target.
@param type The Type of threat level to use. 1 refers to scaled relative threat
(1-100%), 2 refers to raw relative threat (1-130%, like Omen uses).
@return The amount of threat the player has against the target.
--]]
local function getThreat(type)
    _, _, scaledPercent, rawPercent, _ = UnitDetailedThreatStatus("player", "target");
    if (type == 1) then
        return (scaledPercent * 100);
    else
        return (rawPercent * 100);
    end;
end;

---
--[[ Returns the amount of threat that the player has against their target
using relative threat. Using relative threat, the pull level at any range is
100%.
@return The percentage of scaled threat that the player has.
--]]
function FSHelper:ScaledThreat()
    return getThreat(1);
end;

---
--[[ Returns the amount of threat that the player has against their target 
using raw threat. Using raw threat, the pull level is 110% in melee range and
130% at range.
@return The percentage of raw threat that the player has.
--]]
function FSHelper:RawThreat()
    return getThreat(2);
end;

---
--[[ Returns the creature type of the player's target
@return Localized name of the target's creature type (e.g. Beast, Humanoid)
--]]
function FSHelper:TargetType()
    return UnitCreatureType("target");
end;

---
--[[ Checks the player's target level to see if it is a raid boss.
@return 1 if the target is a raid boss, nil otherwise (1nil)
--]]
function FSHelper:TargetIsBoss()
    if UnitLevel("target") == -1 then
        return 1;
    else
        return nil;
    end;
end;

function FSHelper:MxBuff()
end;

function FSHelper:MxDebuff()
end;

function FSHelper:MxBuffByMe()
end;

function FSHelper:MxDebuffByMe()
end;